//*----------------------------------------------------------------------------
//*         ATMEL Microcontroller Software Support  -  ROUSSET  -
//*----------------------------------------------------------------------------
//* The software is delivered "AS IS" without warranty or condition of any
//* kind, either express, implied or statutory. This includes without
//* limitation any warranty or condition with respect to merchantability or
//* fitness for any particular purpose, or against the infringements of
//* intellectual property rights of others.
//*----------------------------------------------------------------------------
//* File Name           : routine_Mayn.c
//* Object              : Mayn application written in C
//* Creation            : S.CADENE   2nd November 2004
//* Vers                : V1.0
//* Aim                 : Start the RTT Peripheral, get out an external signal
//*                       representative of each RTT event through a PIO line,
//*                       start a frequency compensation.
//*----------------------------------------------------------------------------

// Include Standard files
#include "Board.h"

#define         Mayn_frequency    18432000
#define         count_period      (16*Mayn_frequency)
#define         compens_delay     60  // delay in second between comsecutive compensations

//*--------------------------------------------------------------------------------------
//* Function Name       : RTPPRES_Update
//* Object              : Software entry point
//* Input Parameters    : Delay for the next compensation.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
void RTPPRES_Correction (int next_compensation_delay)
{

  AT91PS_PMC     pPMC = AT91C_BASE_PMC;

  unsigned int freq_rc, n;

/* FIRST STEP: We are going to start the embedded PMC process called Timing Interval Acquisition
in application note. For that, we have to stop and start the Mayn Oscillator */
  	// Switch to Slow
        pPMC->PMC_MCKR = (0x4 | AT91C_PMC_CSS_SLOW_CLK);
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Modify PRES value down to 0
        pPMC->PMC_MCKR = AT91C_PMC_CSS_SLOW_CLK;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Stop the PLL
        pPMC->PMC_PLLR = 0;

        // Stop the Main Oscillator
        pPMC->PMC_MOR = 0;

        // Start the Main Oscillator
        pPMC->PMC_MOR = (( AT91C_CKGR_OSCOUNT & (0x06 <<8) | AT91C_CKGR_MOSCEN ));

        // Waiting necessary Main Oscillator startup time
        while(!(pPMC->PMC_SR & AT91C_PMC_MOSCS));

        // Switch to the Main Oscillator clock
        pPMC->PMC_MCKR = AT91C_PMC_CSS_MAIN_CLK;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Start the PLL and waiting necessary settling time
        // The PLLCOUNT value has been calculated according to the following parameters:
        /* PLL Filter components R=1.5k, C1=10 nF, C2=1nF, Fn=18kHz,
        Settling Time (max)= 855 s, RC Oscillator (max)=42 kHz      */
        pPMC->PMC_PLLR = ((AT91C_CKGR_DIV & 0x0E) |
                         (AT91C_CKGR_PLLCOUNT & (0x28 << 8)) |
                         (AT91C_CKGR_MUL & (0x48 << 16)));
        while(!(pPMC->PMC_SR & AT91C_PMC_LOCK));

        // Switch to the PLL Clock
        // Set the PRES field to divide the PLL output by  2
		pPMC->PMC_MCKR = AT91C_PMC_PRES_CLK_2 ;
        while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

        // Switch to PLL output. After this step we're running from the PLL
        pPMC->PMC_MCKR = ((pPMC->PMC_MCKR)|AT91C_PMC_CSS_PLL_CLK);
         while(!(pPMC->PMC_SR & AT91C_PMC_MCKRDY));

/* SECOND STEP: Evaluating real RC-Oscillator working frequency according to the Main Oscillator */

         freq_rc =  (count_period/((*AT91C_CKGR_MCFR & AT91C_CKGR_MAINF)));

        /* Waiting new end of cycle before to modify the RTPPRES field vs. MAINF value
         (take a look at dedicated ATMEL' s Application Note)
        !! IMPORTANT !!: Because there is this constraint, this task should be used in interrupt mode
        In this example, we're using pulling method (It is not the best choice, of course)
        to wait next event just before upgrade RTT prescaler value */
        n = *AT91C_RTTC_RTVR;

        while (n == *AT91C_RTTC_RTVR);
//			    	/ \
//			    	 |
//			The user has to take care in having the smallest delay between the end of the while
//			and the *AT91C_RTTC_RTMR modification
//			    	 |
//			    	\ /
        *AT91C_RTTC_RTMR  =freq_rc;

        // Preset the next compensation event
        *AT91C_RTTC_RTAR = (n) + next_compensation_delay;
}

//*--------------------------------------------------------------------------------------
//* Function Name       : Mayn
//* Object              : Software entry point
//* Input Parameters    : none.
//* Output Parameters   : none.
//*--------------------------------------------------------------------------------------
int main()
{
        RTPPRES_Correction (compens_delay);

        while(1)
        {
            // Second Step:
            if (((*AT91C_RTTC_RTSR) & AT91C_RTTC_ALMS) == AT91C_RTTC_ALMS) RTPPRES_Correction (compens_delay);

         }
}
//* End
